<?php

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if( ! class_exists( 'MywpAbstractSettingModule' ) ) {
  return false;
}

if ( ! class_exists( 'MywpSettingScreenWPAUC_Importer' ) ) :

final class MywpSettingScreenWPAUC_Importer extends MywpAbstractSettingModule {

  static protected $id = 'wpauc_importer_setting';

  static protected $priority = 1000;

  static private $menu = 'wpauc_importer';

  static private $post_type = '';

  static public $errors = array();

  public static function mywp_setting_screens( $setting_screens ) {

    $setting_screens[ self::$id ] = array(
      'title' => __( 'WP Admin UI Customize Importer' ),
      'menu' => self::$menu,
      'controller' => '',
      'use_advance' => false,
      'use_form' => false,
    );

    return $setting_screens;

  }

  private static function wpauc_configs() {

    $wpauc_configs = array(
      'user_roles' => array(
        'title' => __( 'User Roles' ),
        'description' => 'This feature is Add-on. <a href="https://mywpcustomize.com/add_ons/add-on-select-user-roles/" target="_blank">My WP Add-on Select User Roles</a>',
      ),
      'site' => array(
        'title' => sprintf(
          'From WP Admin UI Customize <strong>%s</strong> &gt; To My WP Customize <strong>%s</strong>',
          __( 'Frontend' , 'wp-admin-ui-customize' ),
          '[' . __( 'Frontend' , 'my-wp' ) . '] '. __( 'General' )
        ),
        'can_importer' => true,
      ),
      'admin_general' => array(
        'title' => sprintf(
          'From WP Admin UI Customize <strong>%s</strong> &gt; To My WP Customize <strong>%s</strong>',
          __( 'General' ),
          '[' . __( 'Admin' , 'my-wp' ) . '] '. __( 'General' )
        ),
        'can_importer' => true,
      ),
      'dashboard' => array(
        'title' => sprintf(
          'From WP Admin UI Customize <strong>%s</strong> &gt; To My WP Customize <strong>%s</strong>',
          __( 'Dashboard' ),
          '[' . __( 'Admin' , 'my-wp' ) . '] '. __( 'Dashboard' )
        ),
        'can_importer' => true,
      ),
      'admin_bar_menu' => array(
        'title' => sprintf(
          'From WP Admin UI Customize <strong>%s</strong> &gt; To My WP Customize <strong>%s</strong>',
          __( 'Admin bar' , 'wp-admin-ui-customize' ),
          '[' . __( 'Admin' , 'my-wp' ) . '] '. __( 'Toolbar' , 'my-wp' )
        ),
        'can_importer' => true,
      ),
      'sidemenu' => array(
        'title' => sprintf(
          'From WP Admin UI Customize <strong>%s</strong> &gt; To My WP Customize <strong>%s</strong>',
          __( 'Sidebar' , 'wp-admin-ui-customize' ),
          '[' . __( 'Admin' , 'my-wp' ) . '] '. __( 'Sidebar' , 'my-wp' )
        ),
        'can_importer' => true,
      ),
      'manage_metabox' => array(
        'title' => sprintf(
          'From WP Admin UI Customize <strong>%s</strong> &gt; To My WP Customize <strong>%s</strong>',
          __( 'Management of Meta boxes' , 'wp-admin-ui-customize' ) . '&' . __( 'Posts and Pages' , 'wp-admin-ui-customize' ),
          '[' . __( 'Admin' , 'my-wp' ) . '] '. __( 'Edit Post' ) . '/' . _x( 'Add New' , 'post' )
        ),
        'can_importer' => true,
      ),
      'appearance_menus' => array(
        'title' => sprintf(
          'From WP Admin UI Customize <strong>%s</strong> &gt; To My WP Customize <strong>%s</strong>',
          __( 'Appearance Menus' , 'wp-admin-ui-customize' ),
          '[' . __( 'Admin' , 'my-wp' ) . '] '. __( 'Nav Menus' , 'my-wp' )
        ),
        'can_importer' => true,
      ),
      'login_form' => array(
        'title' => sprintf(
          'From WP Admin UI Customize <strong>%s</strong> &gt; To My WP Customize <strong>%s</strong>',
          __( 'Login Form' , 'wp-admin-ui-customize' ),
          '[' . __( 'Log in' , 'my-wp' ) . '] '. __( 'General' )
        ),
        'can_importer' => true,
      ),
      'plugin_cap' => array(
        'title' => sprintf( __( '%1$s of %2$s %3$s' , 'wp-admin-ui-customize' ) , __( 'Change' ) , __( 'Plugin' ) , __( 'Capabilities' ) ),
        'description' => 'Please change from functions.php of your using theme. e.g.) <code>add_filter( "mywp_manager_capability" , "custom_mywp_manager_capability" );</code>.',
      ),
    );

    $defaults = array(
      'title' => '',
      'can_importer' => false,
      'description' => '',
    );

    foreach( $wpauc_configs as $key => $wpauc_config ) {

      $wpauc_configs[ $key ] = wp_parse_args( $wpauc_config , $defaults );

    }

    return $wpauc_configs;

  }

  private static function get_wpauc_record_key( $key ) {

    global $wauc;

    $key = strip_tags( $key );

    if( isset( $wauc->Record[ $key ] ) ) {

      return $wauc->Record[ $key ];

    }

    return false;

  }

  public static function get_wpauc_record_data( $key ) {

    global $wauc;

    $record_key = self::get_wpauc_record_key( $key );

    $record_option = get_option( $record_key );

    if( isset( $record_option['UPFN'] ) ) {

      unset( $record_option['UPFN'] );

    }

    if( ! empty( $record_option ) ) {

      return $record_option;

    }

    return false;

  }

  public static function mywp_ajax() {

    if( ! MywpApi::is_manager() ) {

      return false;

    }

    add_action( 'wp_ajax_' . MywpSetting::get_ajax_action_name( static::$id , 'do_importer' ) , array( __CLASS__ , 'do_importer' ) );

  }

  public static function do_importer() {

    $action_name = MywpSetting::get_ajax_action_name( self::$id , 'do_importer' );

    if( empty( $_POST[ $action_name ] ) ) {

      return false;

    }

    check_ajax_referer( $action_name , $action_name );

    if( empty( $_POST['importer_item'] ) ) {

      return false;

    }

    $importer_item = strip_tags( $_POST['importer_item'] );

    $importer_item = str_replace( 'item_' , '' , $importer_item );

    $wpauc_configs = self::wpauc_configs();

    if( ! isset( $wpauc_configs[ $importer_item ] ) ) {

      return false;

    }

    if( ! $wpauc_configs[ $importer_item ]['can_importer'] ) {

      wp_send_json_error( array( 'results' => 'This item is can not be used.' ) );

    }

    $results = false;

    if( $importer_item === 'site' ) {

      $results = self::importer_frontend_general();

    } elseif( $importer_item === 'admin_general' ) {

      $results = self::importer_admin_general();

    } elseif( $importer_item === 'admin_bar_menu' ) {

      $results = self::importer_admin_toolbar();

    } elseif( $importer_item === 'sidemenu' ) {

      $results = self::importer_admin_sidebar();

    } elseif( $importer_item === 'dashboard' ) {

      $results = self::importer_admin_dashboard();

    } elseif( $importer_item === 'manage_metabox' ) {

      $results = self::importer_admin_post_edit();

    } elseif( $importer_item === 'appearance_menus' ) {

      $results = self::importer_admin_nav_menu();

    } elseif( $importer_item === 'login_form' ) {

      $results = self::importer_login_general();

    }

    if( empty( $results['success'] ) ) {

      wp_send_json_error( array( 'results' => $results['message'] ) );

    }

    wp_send_json_success( array( 'results' => $results['message'] ) );

  }

  public static function mywp_current_admin_print_styles() {

?>
<style>
#mywp-wpauc-importer {
  margin: 0 auto 30px auto;
}
#mywp-wpauc-importer .importer-item {
  margin: 0 auto 10px 12px;
}
#mywp-wpauc-importer .importer-item .importer-description {
  margin: 4px auto 8px 26px;
  color: #999;
}
#mywp-wpauc-importer .importer-item .results {
  background: #fff;
  padding: 12px;
  margin: 4px auto 12px 26px;
  display: none;
}
#mywp-wpauc-importer .importer-item .results.caution {
  background: #fbd8d8;
  border: 0 none;
}
#mywp-wpauc-importer .importer-item .results.success {
  background: #eaf6ff;
  border: 0 none;
}
#mywp-wpauc-importer .importer-item .results .note {
  color: #cc0000;
}
</style>
<?php

  }

  public static function mywp_current_admin_print_footer_scripts() {

?>
<script>
jQuery(document).ready(function($){

  var all_check_toggle = false;

  $('#all-check').on('click', function() {

    if( all_check_toggle ) {

      all_check_toggle = false;

    } else {

      all_check_toggle = true;

    }

    $('#mywp-wpauc-importer .importer-item .importer-item-check').prop('checked', all_check_toggle);

  });

  $('#do-mywp-wpauc-importer').on('click', function() {

    var $mywp_wpauc_importer = $('#mywp-wpauc-importer');

    $mywp_wpauc_importer.find('.importer-item').each( function( index , el) {

      var $importer_item = $(el);

      var $checkbox = $importer_item.find('.importer-item-check');

      var $results = $importer_item.find('.results');

      if( $checkbox.prop('checked') ) {

        PostData = {
          action: '<?php echo MywpSetting::get_ajax_action_name( self::$id , 'do_importer' ); ?>',
          <?php echo MywpSetting::get_ajax_action_name( self::$id , 'do_importer' ); ?>: '<?php echo wp_create_nonce( MywpSetting::get_ajax_action_name( self::$id , 'do_importer' ) ); ?>',
          importer_item: $checkbox.prop('id')
        };

        $importer_item.find('.spinner').css('visibility', 'visible');

        $results.html('').show().removeClass('caution success');

        $.ajax({
          type: 'post',
          url: ajaxurl,
          data: PostData
        }).done( function( xhr ) {

          $importer_item.find('.spinner').css('visibility', 'hidden');

          if( typeof xhr !== 'object' || xhr.success === undefined ) {

            $results.html( 'An error has occurred. Please try again.' ).addClass('caution');

            return false;

          }

          if( xhr.data.results === undefined ) {

            $results.html( 'An error has occurred. Please try again.' ).addClass('caution');

            return false;

          }

          $results.html( xhr.data.results );

          if( xhr.success ) {

            $results.addClass('success');

          } else {

            $results.addClass('caution');

          }

        }).fail( function( xhr ) {

          $results.html( 'An error has occurred. Please try again.' ).addClass('caution');

          $importer_item.find('.spinner').css('visibility', 'hidden');

        });

      }

    });

  });

});
</script>
<?php

  }

  public static function mywp_current_setting_screen_content() {

    if( ! self::is_work_check() ) {

      foreach( self::$errors as $error ) {

        printf( '<p>%s</p>' , $error );

      }

      return false;

    }

    if( ! MywpExtendsWPAUC_Importer_AdminToolbar::set_default_toolbar_transient() ) {

      echo '<div class="error">';

      foreach( self::$errors as $error ) {

        printf( '<p>%s</p>' , $error );

      }

      echo '</div>';

    }

    if( ! MywpExtendsWPAUC_Importer_AdminSidebar::set_default_sidebar_transient() ) {

      echo '<div class="error">';

      foreach( self::$errors as $error ) {

        printf( '<p>%s</p>' , $error );

      }

      echo '</div>';

    }

    $wpauc_configs = self::wpauc_configs();

    ?>
    <h3 class="mywp-setting-screen-subtitle"><?php _e( 'From WP Admin UI Customize To My WP Customize' , 'my-wp' ); ?></h3>

    <p>Please choose the feature you want to duplicate.</p>

    <p><button type="button" class="button button-secondary" id="all-check"><?php _e( 'All Check' , 'my-wp' ); ?></button></p>

    <div id="mywp-wpauc-importer">

      <?php foreach( $wpauc_configs as $wpauc_key => $wpauc_config ) : ?>

        <?php if( ! $wpauc_config['can_importer']  ) : ?>

          <?php continue; ?>

        <?php endif; ?>

        <div class="importer-item">

          <label>
            <input type="checkbox" class="importer-item-check" id="item_<?php echo esc_attr( $wpauc_key ); ?>" <?php disabled( $wpauc_config['can_importer'] , false ); ?> />
            <?php echo $wpauc_config['title']; ?>
            <span class="spinner"></span>

          </label>

          <?php if( ! empty( $wpauc_config['description'] ) ) : ?>

            <div class="importer-description"><?php echo $wpauc_config['description']; ?></div>

          <?php endif; ?>

          <div class="results"></div>

        </div>

      <?php endforeach; ?>

    </div>

    <p><button type="button" class="button button-primary" id="do-mywp-wpauc-importer">Run</button></p>

    <p>&nbsp;</p>

    <p>Below features can not be Importer.</p>

    <ul>

      <?php foreach( $wpauc_configs as $wpauc_key => $wpauc_config ) : ?>

        <?php if( $wpauc_config['can_importer']  ) : ?>

          <?php continue; ?>

        <?php endif; ?>

        <li>

          <?php echo $wpauc_config['title']; ?>

          <?php if( ! empty( $wpauc_config['description'] ) ) : ?>

            <div class="importer-description"><?php echo $wpauc_config['description']; ?></div>

          <?php endif; ?>

        </li>

      <?php endforeach; ?>

    </ul>

    <p>&nbsp;</p>
    <?php

  }



  private static function is_work_check() {

    $is_wauc_activate = MywpThirdparty::is_plugin_activate( MywpThirdpartyModuleWPAUC::get_base_name() );

    if( ! $is_wauc_activate ) {

      self::$errors[] = sprintf( '%1$s can not work because %2$s is not activated. <strong>Please activated to %2$s</strong>.' , MYWP_WAUC_MULTIPLE_ADDON_IMPORTER_NAME , 'WP Admin UI Customize.' );

      return false;

    }

    return true;

  }

  public static function change_shortcode( $string ) {

    if( empty( $string ) ) {

      return $string;

    }

    if( strpos( $string , '[site_name]' ) !== false ) {

      $string = str_replace( '[site_name]' , '[mywp_site site_id="1" field="name"]' , $string );

    }

    if( strpos( $string , '[site_url]' ) !== false ) {

      $string = str_replace( '[site_url]' , '[mywp_url site_id="1" site="1"]' , $string );

    }

    if( strpos( $string , '[blog_name]' ) !== false ) {

      $string = str_replace( '[blog_name]' , '[mywp_site field="name"]' , $string );

    }

    if( strpos( $string , '[blog_url]' ) !== false ) {

      $string = str_replace( '[blog_url]' , '[mywp_url]' , $string );

    }

    if( strpos( $string , '[template_directory_uri]' ) !== false ) {

      $string = str_replace( '[template_directory_uri]' , '[mywp_theme field="url" parent="1"]' , $string );

    }

    if( strpos( $string , '[stylesheet_directory_uri]' ) !== false ) {

      $string = str_replace( '[stylesheet_directory_uri]' , '[mywp_theme field="url"]' , $string );

    }

    if( strpos( $string , '[update_total]' ) !== false ) {

      $string = str_replace( '[update_total]' , '[mywp_update_count]' , $string );

    }

    if( strpos( $string , '[update_total_format]' ) !== false ) {

      $string = str_replace( '[update_total_format]' , '[mywp_update_count tag="1"]' , $string );

    }

    if( strpos( $string , '[update_plugins]' ) !== false ) {

      $string = str_replace( '[update_plugins]' , '[mywp_update_count type="plugins"]' , $string );

    }

    if( strpos( $string , '[update_plugins_format]' ) !== false ) {

      $string = str_replace( '[update_plugins_format]' , '[mywp_update_count tag="1" type="plugins"]' , $string );

    }

    if( strpos( $string , '[update_themes]' ) !== false ) {

      $string = str_replace( '[update_themes]' , '[mywp_update_count type="themes"]' , $string );

    }

    if( strpos( $string , '[update_themes_format]' ) !== false ) {

      $string = str_replace( '[update_themes_format]' , '[mywp_update_count tag="1" type="themes"]' , $string );

    }

    if( strpos( $string , '[comment_count]' ) !== false ) {

      $string = str_replace( '[comment_count]' , '[mywp_comment_count]' , $string );

    }

    if( strpos( $string , '[comment_count_format]' ) !== false ) {

      $string = str_replace( '[comment_count_format]' , '[mywp_comment_count tag="1"]' , $string );

    }

    if( strpos( $string , '[user_name]' ) !== false ) {

      $string = str_replace( '[user_name]' , '[mywp_user field="displayname"]' , $string );

    }

    if( strpos( $string , '[user_login_name]' ) !== false ) {

      $string = str_replace( '[user_login_name]' , '[mywp_user field="user_login"]' , $string );

    }

    if( strpos( $string , '[user_avatar]' ) !== false ) {

      $string = str_replace( '[user_avatar]' , '[mywp_user field="avatar" size="16"]' , $string );

    }

    if( strpos( $string , '[user_avatar_64]' ) !== false ) {

      $string = str_replace( '[user_avatar_64]' , '[mywp_user field="avatar" size="64"]' , $string );

    }

    if( strpos( $string , '[post_type]' ) !== false ) {

      $string = str_replace( '[post_type]' , '' , $string );

    }

    if( strpos( $string , '[woocommerce_order_process_count]' ) !== false ) {

      $string = str_replace( '[woocommerce_order_process_count]' , '[mywp_woocommerce_order_count tag="1"]' , $string );

    }

    return $string;

  }

  public static function mywp_model_get_option_key_mywp_admin_post_edit( $option_key ) {

    if( empty( self::$post_type ) ) {

      return $option_key;

    }

    $option_key .= '_' . self::$post_type;

    return $option_key;

  }

  public static function add_post( $post_type , $args = array() , $post_metas = array() , $post_terms = array() ) {

    global $wpdb;

    $default_args = array(
      'post_type' => $post_type,
      'post_status' => 'publish',
      'post_parent' => 0,
    );

    $post = wp_parse_args( $args , $default_args );

    $post_id = wp_insert_post( $post );

    if ( empty( $post_id ) or is_wp_error( $post_id ) ) {

      return $post_id;

    }

    if( ! empty( $post_terms ) ) {

      foreach( $post_terms as $post_term ) {

        if( empty( $post_term['taxonomy'] ) or empty( $post_term['term'] ) ) {

          continue;

        }

        $taxonomy = strip_tags( $post_term['taxonomy'] );

        $term = strip_tags( $post_term['term'] );

        $term_exists = term_exists( $term , $taxonomy );

        if( empty( $term_exists ) ) {

          wp_insert_term( $term , $taxonomy );

        }

        wp_set_object_terms( $post_id , $term , $taxonomy );

      }

    }

    if( ! empty( $post_metas ) ) {

      $add_meta_data = array();

      foreach( $post_metas as $meta_key => $meta_value ) {

        $meta_key = strip_tags( $meta_key );

        $add_meta_data[] = $wpdb->prepare( "(NULL, %d, %s, %s)" , $post_id , wp_unslash( $meta_key ) , maybe_serialize( wp_unslash( $meta_value ) ) );
      }

      $query = "INSERT INTO $wpdb->postmeta (meta_id, post_id, meta_key, meta_value) VALUES " . join( ',' , $add_meta_data );

      $wpdb->query( $query );

    }

    return $post_id;

  }

  private static function importer_frontend_general() {

    $results = array( 'success' => false , 'message' => '' );

    $wpauc_setting_data = self::get_wpauc_record_data( 'site' );

    if( empty( $wpauc_setting_data ) ) {

      $results['success'] = true;

      $results['message'] = 'WP Admin UI Customize Setting data not found.';

      return $results;

    }

    $setting_screen_id = 'frontend_general';

    $mywp_model = MywpSetting::get_model( $setting_screen_id );

    if( empty( $mywp_model ) ) {

      $results['message'] = '$mywp_model not found.';

      return $results;

    }

    $setting_data = $mywp_model->get_default_data();

    if( ! empty( $wpauc_setting_data['wp_generator'] ) ) {

      $setting_data['hide_wp_generator'] = true;

    }

    if( ! empty( $wpauc_setting_data['wlwmanifest_link'] ) ) {

      $setting_data['hide_wlwmanifest_link'] = true;

    }

    if( ! empty( $wpauc_setting_data['rsd_link'] ) ) {

      $setting_data['hide_rsd_link'] = true;

    }

    if( ! empty( $wpauc_setting_data['feed_links'] ) ) {

      $setting_data['hide_feed_links'] = true;

    }

    if( ! empty( $wpauc_setting_data['feed_links_extra'] ) ) {

      $setting_data['hide_feed_links_extra'] = true;

    }

    if( ! empty( $wpauc_setting_data['admin_bar'] ) ) {

      if( $wpauc_setting_data['admin_bar'] === 'hide' ) {

        $setting_data['admin_bar'] = 'hide';

      } elseif( $wpauc_setting_data['admin_bar'] === 'front' ) {

        $results['message'] .= '<p class="note">Sorry, My WP Customize is not possible the Admin Bar to Frontend.</p>';

      }

    }

    $results['success'] = true;

    $results['message'] .= 'Success!';

    $mywp_model->update_data( $setting_data );

    return $results;

  }

  private static function importer_admin_general() {

    $results = array( 'success' => false , 'message' => '' );

    $wpauc_setting_data = self::get_wpauc_record_data( 'admin_general' );

    if( empty( $wpauc_setting_data ) ) {

      $results['success'] = true;

      $results['message'] = 'WP Admin UI Customize Setting data not found.';

      return $results;

    }

    $setting_screen_id = 'admin_general';

    $mywp_model = MywpSetting::get_model( $setting_screen_id );

    if( empty( $mywp_model ) ) {

      $results['message'] = '$mywp_model not found.';

      return $results;

    }

    $setting_data = $mywp_model->get_default_data();

    if( ! empty( $wpauc_setting_data['notice_update_core'] ) ) {

      $setting_data['hide_update_notice']['core'] = true;

    }

    if( ! empty( $wpauc_setting_data['notice_update_plugin'] ) ) {

      $setting_data['hide_update_notice']['plugins'] = true;

    }

    if( ! empty( $wpauc_setting_data['notice_update_theme'] ) ) {

      $setting_data['hide_update_notice']['themes'] = true;

    }

    if( ! empty( $wpauc_setting_data['screen_option_tab'] ) ) {

      $setting_data['hide_screen_tabs']['options'] = true;

    }

    if( ! empty( $wpauc_setting_data['help_tab'] ) ) {

      $setting_data['hide_screen_tabs']['help'] = true;

    }

    if( ! empty( $wpauc_setting_data['footer_text'] ) ) {

      $setting_data['custom_footer_text'] = self::change_shortcode( $wpauc_setting_data['footer_text'] );

      $setting_data['hide_footer_text']['left'] = true;

    }

    if( ! empty( $wpauc_setting_data['css'] ) ) {

      $setting_data['include_css_file'] = self::change_shortcode( $wpauc_setting_data['css'] );

    }

    if( ! empty( $wpauc_setting_data['title_tag'] ) ) {

      $setting_data['hide_core_title_tag'] = true;

    }

    if( ! empty( $wpauc_setting_data['resize_admin_bar'] ) ) {

      $results['message'] .= '<p class="note">Sorry, My WP Customize is not possible the Resize Admin Bar.</p>';

    }

    $setting_data['advance'] = true;

    $results['success'] = true;

    $results['message'] .= 'Success!';

    $mywp_model->update_data( $setting_data );

    return $results;

  }

  private static function importer_admin_toolbar() {

    $results = array( 'success' => false , 'message' => '' );

    $wpauc_setting_data = self::get_wpauc_record_data( 'admin_bar_menu' );

    if( empty( $wpauc_setting_data ) ) {

      $results['success'] = true;

      $results['message'] = 'WP Admin UI Customize Setting data not found.';

      return $results;

    }

    $setting_screen_id = 'admin_toolbar';

    $mywp_model = MywpSetting::get_model( $setting_screen_id );

    if( empty( $mywp_model ) ) {

      $results['message'] = '$mywp_model not found.';

      return $results;

    }

    $default_toolbar_items = MywpExtendsWPAUC_Importer_AdminToolbar::get_transient_default_admin_toolbar();

    if( empty( $default_toolbar_items ) ) {

      $results['message'] = '$default_toolbar_items not found. Please reload screen and try again.';

      return $results;

    }

    $setting_data = $mywp_model->get_default_data();

    MywpExtendsWPAUC_Importer_AdminToolbar::delete_current_toolbar_items();

    MywpExtendsWPAUC_Importer_AdminToolbar::toolbar_regist( 'left' , '' );

    MywpExtendsWPAUC_Importer_AdminToolbar::toolbar_regist( 'right' , '' );

    $setting_data['advance'] = true;

    $results['success'] = true;

    if( ! empty( self::$errors ) ) {

      foreach( self::$errors as $error_message ) {

        $results['message'] .= sprintf( '%s <br />' , $error_message );

      }

    }

    $results['message'] .= 'Success!';

    $mywp_model->update_data( $setting_data );

    return $results;

  }

  private static function importer_admin_sidebar() {

    $results = array( 'success' => false , 'message' => '' );

    $wpauc_setting_data = self::get_wpauc_record_data( 'sidemenu' );

    if( empty( $wpauc_setting_data ) ) {

      $results['success'] = true;

      $results['message'] = 'WP Admin UI Customize Setting data not found.';

      return $results;

    }

    $setting_screen_id = 'admin_sidebar';

    $mywp_model = MywpSetting::get_model( $setting_screen_id );

    if( empty( $mywp_model ) ) {

      $results['message'] = '$mywp_model not found.';

      return $results;

    }

    $default_sidebar_items = MywpExtendsWPAUC_Importer_AdminSidebar::get_transient_default_admin_sidebar();

    if( empty( $default_sidebar_items ) ) {

      $results['message'] = '$default_sidebar_items not found. Please reload screen and try again.';

      return $results;

    }

    $setting_data = $mywp_model->get_default_data();

    MywpExtendsWPAUC_Importer_AdminSidebar::delete_current_sidebar_items();

    MywpExtendsWPAUC_Importer_AdminSidebar::sidebar_regist( '' );

    $setting_data['advance'] = true;

    $results['success'] = true;

    if( ! empty( self::$errors ) ) {

      foreach( self::$errors as $error_message ) {

        $results['message'] .= sprintf( '%s <br />' , $error_message );

      }

    }

    $results['message'] .= 'Success!';

    $mywp_model->update_data( $setting_data );

    return $results;

  }

  private static function importer_admin_dashboard() {

    $results = array( 'success' => false , 'message' => '' );

    $wpauc_setting_data = self::get_wpauc_record_data( 'dashboard' );

    if( empty( $wpauc_setting_data ) ) {

      $results['success'] = true;

      $results['message'] = 'WP Admin UI Customize Setting data not found.';

      return $results;

    }

    $setting_screen_id = 'admin_dashboard';

    $mywp_model = MywpSetting::get_model( $setting_screen_id );

    if( empty( $mywp_model ) ) {

      $results['message'] = '$mywp_model not found.';

      return $results;

    }

    $setting_data = $mywp_model->get_default_data();

    if( ! empty( $wpauc_setting_data['show_welcome_panel'] ) ) {

      $setting_data['hide_welcome_panel'] = true;

      unset( $wpauc_setting_data['show_welcome_panel'] );

    }

    if( ! empty( $wpauc_setting_data ) ) {

      foreach( $wpauc_setting_data as $wpauc_metabox_id => $wpauc_metabox_setting ) {

        if( empty( $wpauc_metabox_setting ) ) {

          continue;

        }

        if( ! is_array( $wpauc_metabox_setting ) ) {

          continue;

        }

        $setting_data['meta_boxes'][ $wpauc_metabox_id ] = array( 'action' => '' , 'title' => '' );

        if( ! empty( $wpauc_metabox_setting['remove'] ) ) {

          $setting_data['meta_boxes'][ $wpauc_metabox_id ]['action'] = 'remove';

        }

        if( ! empty( $wpauc_metabox_setting['name'] ) ) {

          $setting_data['meta_boxes'][ $wpauc_metabox_id ]['title'] = wp_unslash( $wpauc_metabox_setting['name'] );

        }

      }

    }

    if( ! empty( $wpauc_setting_data['metabox_move'] ) ) {

      $setting_data['prevent_meta_box'] = true;

    }

    $setting_data['advance'] = true;

    $results['success'] = true;

    $results['message'] .= 'Success!';

    $mywp_model->update_data( $setting_data );

    return $results;

  }

  private static function importer_admin_post_edit() {

    $results = array( 'success' => false , 'message' => '' );

    $wpauc_setting_data = self::get_wpauc_record_data( 'manage_metabox' );

    if( empty( $wpauc_setting_data ) ) {

      $results['success'] = true;

      $results['message'] = 'WP Admin UI Customize Setting data not found.';

      return $results;

    }

    $wpauc_setting_data_post_add_edit = self::get_wpauc_record_data( 'post_add_edit' );

    $setting_screen_id = 'admin_post_edit';

    $post_metabox_settings = array();

    if( ! empty( $wpauc_setting_data ) ) {

      foreach( $wpauc_setting_data as $post_type => $wpauc_metaboxes_setting ) {

        if( empty( $wpauc_metaboxes_setting ) ) {

          continue;

        }

        if( ! is_array( $wpauc_metaboxes_setting ) ) {

          continue;

        }

        self::$post_type = $post_type;

        add_filter( 'mywp_model_get_option_key_mywp_admin_post_edit' , array( __CLASS__ , 'mywp_model_get_option_key_mywp_admin_post_edit' ) );

        $mywp_model = MywpSetting::get_model( $setting_screen_id );

        if( empty( $mywp_model ) ) {

          $results['message'] = '$mywp_model not found.';

          return $results;

        }

        $setting_data = $mywp_model->get_default_data();

        foreach( $wpauc_metaboxes_setting as $wpauc_metabox_id => $wpauc_metabox_setting ) {

          $setting_data['meta_boxes'][ $wpauc_metabox_id ] = array( 'action' => '' , 'title' => '' );

          if( ! empty( $wpauc_metabox_setting['remove'] ) ) {

            $setting_data['meta_boxes'][ $wpauc_metabox_id ]['action'] = 'remove';

          }

          if( ! empty( $wpauc_metabox_setting['name'] ) ) {

            $setting_data['meta_boxes'][ $wpauc_metabox_id ]['title'] = wp_unslash( $wpauc_metabox_setting['name'] );

          }

        }

        if( ! empty( $wpauc_setting_data_post_add_edit['default_permalink'] ) ) {

          $setting_data['hide_change_permalink'] = true;

        }

        if( ! empty( $wpauc_setting_data_post_add_edit['allow_comments'] ) ) {

          if( ! empty( $setting_data['meta_boxes']['commentstatusdiv']['action'] ) ) {

            $setting_data['meta_boxes']['commentstatusdiv']['action'] = 'hide';

          }

          $setting_data['hide_change_permalink'] = true;

        }

        $setting_data['advance'] = true;

        $mywp_model->update_data( $setting_data );

        remove_filter( 'mywp_model_get_option_key_mywp_admin_post_edit' , array( __CLASS__ , 'mywp_model_get_option_key_mywp_admin_post_edit' ) );

      }

    }

    self::$post_type = '';

    remove_filter( 'mywp_model_get_option_key_mywp_admin_post_edit' , array( __CLASS__ , 'mywp_model_get_option_key_mywp_admin_post_edit' ) );

    $results['success'] = true;

    $results['message'] .= '<p class="note">Sorry, My WP Customize is not possible to toggle metabox setting.</p><br />';

    $results['message'] .= 'Success!';

    return $results;

  }

  private static function importer_admin_nav_menu() {

    $results = array( 'success' => false , 'message' => '' );

    $wpauc_setting_data = self::get_wpauc_record_data( 'appearance_menus' );

    if( empty( $wpauc_setting_data ) ) {

      $results['success'] = true;

      $results['message'] = 'WP Admin UI Customize Setting data not found.';

      return $results;

    }

    $setting_screen_id = 'admin_nav_menu';

    $mywp_model = MywpSetting::get_model( $setting_screen_id );

    if( empty( $mywp_model ) ) {

      $results['message'] = '$mywp_model not found.';

      return $results;

    }

    $setting_data = $mywp_model->get_default_data();

    if( ! empty( $wpauc_setting_data['add_new_menu'] ) ) {

      $setting_data['hide_add_new_menu'] = true;

    }

    if( ! empty( $wpauc_setting_data['delete_menu'] ) ) {

      $setting_data['hide_delete_menu'] = true;

    }

    $setting_data['advance'] = true;

    $results['success'] = true;

    $results['message'] .= 'Success!';

    $mywp_model->update_data( $setting_data );

    return $results;

  }

  private static function importer_login_general() {

    $results = array( 'success' => false , 'message' => '' );

    $wpauc_setting_data = self::get_wpauc_record_data( 'loginscreen' );

    if( empty( $wpauc_setting_data ) ) {

      $results['success'] = true;

      $results['message'] = 'WP Admin UI Customize Setting data not found.';

      return $results;

    }

    $setting_screen_id = 'login_general';

    $mywp_model = MywpSetting::get_model( $setting_screen_id );

    if( empty( $mywp_model ) ) {

      $results['message'] = '$mywp_model not found.';

      return $results;

    }

    $setting_data = $mywp_model->get_default_data();

    if( ! empty( $wpauc_setting_data['login_headerurl'] ) ) {

      $setting_data['logo_link_url'] = self::change_shortcode( $wpauc_setting_data['login_headerurl'] );

    }

    if( ! empty( $wpauc_setting_data['login_headertitle'] ) ) {

      $setting_data['logo_title'] = self::change_shortcode( $wpauc_setting_data['login_headertitle'] );

    }

    if( ! empty( $wpauc_setting_data['login_headerlogo'] ) ) {

      $setting_data['logo_image_path'] = self::change_shortcode( $wpauc_setting_data['login_headerlogo'] );

    }

    if( ! empty( $wpauc_setting_data['login_css'] ) ) {

      $setting_data['include_css_file'] = self::change_shortcode( $wpauc_setting_data['login_css'] );

    }

    if( ! empty( $wpauc_setting_data['login_footer'] ) ) {

      $setting_data['custom_footer_text'] = self::change_shortcode( $wpauc_setting_data['login_footer'] );

    }

    $results['success'] = true;

    $results['message'] .= 'Success!';

    $mywp_model->update_data( $setting_data );

    return $results;

  }

}

MywpSettingScreenWPAUC_Importer::init();

endif;
